//完整组件库的打包配置
//webpack.prod.js
const path = require('path')
const webpack = require('webpack')
const {merge} = require('webpack-merge');
const webpackBaseConfig = require('./webpack.base.js');
const {VueLoaderPlugin} = require("vue-loader");
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin


const basePath = path.resolve(__dirname, '../')
function resolve(dir) {
  return path.join(basePath, dir)
}
module.exports = merge(webpackBaseConfig, {
  mode:'none',
  entry: path.join(basePath, 'src', 'enter.js'),
  output: {
    path: path.resolve(__dirname, '../lib'),
    publicPath: '/lib/',
    filename: 'tip-components.min.js',  // 输出文件名
    library: 'tip-components', // 组件库名称
    libraryTarget: 'umd',  //模块化格式
    umdNamedDefine: true
  },
  externals: {
    vue: {  //将vue依赖 "外部化"，不打包进组件库
      root: 'Vue',
      commonjs: 'vue',
      commonjs2: 'vue',
      amd: 'vue'
    }
  },
  resolve: {
    alias: {
      '@': resolve('src'),
    }
  },
  plugins: [
    new VueLoaderPlugin(),
    new BundleAnalyzerPlugin()
  ],
  module: {
    rules: [
      {
        test: /\.vue$/,
        use:  ['vue-loader']
      },
      {
        test: /\.css$/,
        use:  ['style-loader','css-loader','postcss-loader']
      },
      {
        test: /\.s[ac]ss$/i,
        use:  ['style-loader','css-loader','postcss-loader','sass-loader']
      },
      {
        test: /\.js$/,
        loader: 'babel-loader',
        exclude: /node_modules/,
      }
    ]
  },
});