/**
 * 绘制旋转虚线圆, 画布大小继承于外部盒子, 更方便适配
 * */
function drawCircleDash({
                          canvasId,
                          dashLength = 8,
                          dashGap = 4,
                          lineWidth = 2,
                          lineColor = '#aaf2ff',
                          speed = 0.1
                        }) {
  let canvas = document.getElementById(canvasId)
  let ctx = canvas.getContext('2d')
  const width = canvas.width
  const height = canvas.height
  const center_x = width / 2
  const center_y = height / 2
  const radius = center_x - 2 * lineWidth  //与边界留lineWidth的空隙
  let intervalId = setInterval(() => {
    ctx.clearRect(0, 0, width, height)
    ctx.setLineDash([dashLength, dashGap])
    ctx.lineWidth = lineWidth
    ctx.strokeStyle = lineColor
    ctx.beginPath()
    ctx.arc(center_x, center_y, radius, 0, 2 * Math.PI)
    ctx.stroke()
    ctx.translate(center_x, center_y)
    ctx.rotate(Math.PI * speed)
    ctx.translate(-center_x, -center_y)
  }, 100)
  return intervalId
}

function drawProgressCircle({
  canvasId,
  lineWidth = 6,
  backgroundImage,   //绘制背景图
  percent,  //百分比
  label,   //标注文字
  unit,     //标注百分比
  foreColor = '#aaf2ff',    //进度圆环颜色
  fontColor = '#00d9ff',    //标注字体颜色
  percentColor = '#e5f3fe', //百分比颜色
  percentSize = 12,       //百分比字体大小
  lineDash = [10, 4],    //绘制虚线
  counterclockwise = false,  //顺时针
  percentBaseLine = 'bottom',   //进度百分比底部对齐
}) {
  let canvas = document.getElementById(canvasId)
  let ctx = canvas.getContext('2d')
  const width = canvas.width
  const height = canvas.height
  const center_x = width / 2
  const center_y = height / 2
  const radius = center_x - 2 * lineWidth  //与边界留lineWidth的空隙
  let speed = 0    //当前进度
  let rad = Math.PI * 2 / 100

  const img = new Image()
  img.src = backgroundImage
  img.onload = () => {
    (function drawFrame() {
      window.requestAnimationFrame(drawFrame)
      ctx.clearRect(0, 0, width, height)
      backgroundCircle(img)  //背景
      drawLabel(label, unit)  //标注
      drawPercent(speed)    //进度百分比
      foregroundCircle(speed)   //进度圆环
      if (speed >= percent) {
        return speed = percent
      }
      speed += 1
    }())
  }

  //绘制背景图片
  function backgroundCircle(img) {
    ctx.save()
    ctx.drawImage(img, 0, 0, width, height)
    ctx.restore()
  }

  //绘制进度圆环
  function foregroundCircle(n) {
    ctx.save()
    ctx.strokeStyle = foreColor
    ctx.lineWidth = lineWidth
    lineDash ? ctx.setLineDash(lineDash) : ctx.lineCap = 'round'
    ctx.beginPath()
    let endRadius, startRadius = -Math.PI / 2
    if (counterclockwise) {
      //逆时针
      endRadius = -Math.PI / 2 - n * rad
    } else {
      endRadius = -Math.PI / 2 + n * rad
    }
    ctx.arc(center_x, center_y, radius, startRadius, endRadius, counterclockwise)
    ctx.stroke()
    ctx.closePath()
    ctx.restore()
  }

  //绘制文字
  function drawLabel(label, unit) {
    const font_size = percentSize
    ctx.save()
    ctx.fillStyle = fontColor
    let yPosition
    if (percentBaseLine == 'bottom') {
      ctx.textBaseline = 'top'
      yPosition = center_y + percentSize / 2
    } else {
      yPosition = center_y
    }
    ctx.font = font_size + 'px Helvetica'
    const text_width = ctx.measureText(label).width
    ctx.fillText(label, center_x - text_width / 2, yPosition)
    //单位
    if (unit) {
      ctx.fillStyle = percentColor
      ctx.font = percentSize / 1.5 + 'px Helvetica'
      ctx.fillText(unit, center_x + text_width / 2, yPosition)
    }
    ctx.restore()
  }

  //进度百分比
  function drawPercent(n) {
    ctx.save()
    ctx.fillStyle = percentColor
    ctx.font = percentSize + 'px Helvetica'
    const text_width = ctx.measureText(n + '%').width
    ctx.textBaseline = percentBaseLine
    let yPosition
    if (percentBaseLine == 'bottom') {
      yPosition = center_y
    } else {
      yPosition = center_y + percentSize / 2
    }
    ctx.fillText(n + '%', center_x - text_width / 2, yPosition)
    ctx.restore()
  }


}

export default {
  drawCircleDash,
  drawProgressCircle
}