//概览
//加载倾斜(携带请求头)
//加载白膜(渐变效果)/加载拉伸面
//
import {MapboxLayer} from '@deck.gl/mapbox';
import {Tile3DLayer, MVTLayer } from "@deck.gl/geo-layers"
import {Tiles3DLoader} from '@loaders.gl/3d-tiles'
import {PolygonLayer } from "@deck.gl/layers";
import {LayerExtension} from '@deck.gl/core';
import {MVTLoader} from '@loaders.gl/mvt';

import axios from 'axios';
//图层拓展器，用于对Shader进行重写与拓展
class BuildingFilter extends LayerExtension {
  getShaders() {
      return {
          inject: {
              //注入顶点着色器声明
              'vs:#decl': `
                  varying vec2 vPosition;
              `,
              //注入顶点着色器，对varying变量赋值
              'vs:#main-end': `
                  vPosition = vertexPositions;
              `,
              //注入片元着色器声明
              'fs:#decl': `
                  varying vec2 vPosition;
              `,
              //重写颜色绘制函数
              'fs:DECKGL_FILTER_COLOR': `
                  color = vec4(color.xyz, color.w * pow(vPosition.y,2.0));
              `
          }
      };
  }
}

function load3DTiles(layerConfig) {
  let layer = new MapboxLayer({
    id: layerConfig.id,
    type: Tile3DLayer,
    data: layerConfig.url,
    loader: Tiles3DLoader,
    loadOptions: {
      tileset: {
        throttleRequests: false,
      },
      // fetch: {
      //   headers: {"szvsud-license-key": 'yO+ChHjBx7FZmtGQnUVVCVe8JcKfj0ggEmqjUkBrTOEk9EF+j3Nk65pqSo5W8+Qz'}
      // }
    },
    onTileError: (err) => {
      console.log(err)
    },
    onTilesetLoad: (tileset) => {
      // tileset.setProps({ maximumScreenSpaceError: 1 })
      // const { cartographicCenter, zoom } = tileset;
      // this.viewState = {
      //   ...this.viewState,
      //   longitude: cartographicCenter[0],
      //   latitude: cartographicCenter[1],
      //   zoom
      // }
      // let viewState = this.viewState
      // //设置x、y、z偏移，有些倾斜摄影模型加载进来不在该位置
      // //需要进行位置的调整，但我在进行旋转、和比例缩放的时候不起作用
      // //欢迎来指正
      // let mMtx=new core_1.Matrix4().makeTranslation(1,1,-200)
      
      // tileset.modelMatrix=mMtx
      // this.map.jumpTo({
      //   center: [viewState.longitude, viewState.latitude],
      //   zoom: viewState.zoom,
      //   //bearing: viewState.bearing,
      //   //pitch: viewState.pitch,
      // });

    }
  })
  return layer
}

//拉伸渐变面效果
//polygonlayer只支持单面,  多面加载可以选择geojsonlayer
function addExtrutionPolyLayer(options){
  const extrutionPolyLayer = new MapboxLayer({
      id: options.id,
      type: PolygonLayer,
      data: options.data,
      extruded: true,
      pickable: false,
      stroked: true,
      filled: true,
      wireframe: false,
      lineWidthMinPixels: 1,
      getPolygon: d => d.geometry.coordinates[0],
      getElevation: d => d.properties['gd'],
      getFillColor: [0, 249, 217,255],
      getLineColor: [0, 249, 217,255],
      // extensions: [new BuildingFilter()]
  })
  return extrutionPolyLayer
}

//渐变白膜
function addMvtLayer(options){
  const layer = new MapboxLayer({
    id: options.id,
    type: MVTLayer,
    loader: MVTLoader,
    loadOptions: {
      mvt: {
        coordinates: 'wgs84',
        layers:['建筑物']
      }
    },
    data: options.data,
    minzoom: 14.0,
    extruded: true,
    pickable: false,
    stroked: true,
    filled: true,
    wireframe: false,
    lineWidthMinPixels: 1,
    getPolygon: d => d.geometry.coordinates[0],
    getElevation: d => d.properties['gd'],
    getFillColor: [0, 249, 217,255],
    getLineColor: [0, 249, 217,255],
    extensions: [new BuildingFilter()]
  })
  return layer
}
export async function addDeckglLayers(type){
  let layer
  if(type=='3dtiles'){
    //添加cesium格式3dtiles 
    layer = load3DTiles({
      id: "3dTiles-qingxie",
      describe:"倾斜摄影",
      url:'xxxxxx',
      layerType: "3DTiles",
    })
  }else if(type=='extrutionPolyLayer'){
    const {data} = await axios.get('./json/building.json')
    console.log(data)
    layer = addExtrutionPolyLayer({
      id:'extrutionPolyLayer',
      data:data.features
    })
  }else if(type=='mvt'){
    layer = addMvtLayer({
      id:'mvtLayer',
      data:'xxxxxx'
    })
  }
  
  window.$mbmap.addLayer(layer)
}